@@ -3,12 +3,10 @@ package FusionInventory::Agent;
 use strict;
 use warnings;
 
-use Cwd;
 use English qw(-no_match_vars);
 use UNIVERSAL::require;
 use File::Glob;
 use IO::Handle;
-use POSIX ":sys_wait_h"; # WNOHANG
 use Storable 'dclone';
 
 use FusionInventory::Agent::Version;
@@ -19,7 +17,6 @@ use FusionInventory::Agent::Storage;
 use FusionInventory::Agent::Target::Local;
 use FusionInventory::Agent::Target::Server;
 use FusionInventory::Agent::Tools;
-use FusionInventory::Agent::Tools::Generic;
 use FusionInventory::Agent::Tools::Hostname;
 use FusionInventory::Agent::XML::Query::Prolog;
 
@@ -50,7 +47,6 @@ sub new {
         datadir => $params{datadir},
         libdir  => $params{libdir},
         vardir  => $params{vardir},
-        sigterm => $params{sigterm},
         targets => [],
         tasks   => []
     };
@@ -62,7 +58,8 @@ sub new {
 sub init {
     my ($self, %params) = @_;
 
-    my $config = FusionInventory::Agent::Config->new(
+    # Skip create object if still defined (re-init case)
+    my $config = $self->{config} || FusionInventory::Agent::Config->new(
         confdir => $self->{confdir},
         options => $params{options},
     );
@@ -84,17 +81,7 @@ sub init {
     $logger->debug("Storage directory: $self->{vardir}");
     $logger->debug("Lib directory: $self->{libdir}");
 
-    $self->{storage} = FusionInventory::Agent::Storage->new(
-        logger    => $logger,
-        directory => $self->{vardir}
-    );
-
-    # handle persistent state
-    $self->_loadState();
-
-    $self->{deviceid} = _computeDeviceId() if !$self->{deviceid};
-
-    $self->_saveState();
+    $self->_handlePersistentState();
 
     $self->_createTargets();
 
@@ -127,219 +114,50 @@ sub init {
 
     $self->{tasks} = \@tasks;
 
-    if ($config->{daemon}) {
-        $self->_createDaemon();
-    }
-
-    # create HTTP interface
-    if (($config->{daemon} || $config->{service}) && !$config->{'no-httpd'}) {
-        $self->_createHttpInterface();
-    }
-
-    # install signal handler to handle graceful exit
-    $self->_installSignalHandlers();
-
-    $self->{logger}->info("$PROVIDER Agent starting")
-        if $self->{config}->{daemon} || $self->{config}->{service};
-
-    $self->ApplyServiceOptimizations();
-
-    $self->{logger}->info("Options 'no-task' and 'tasks' are both used. Be careful that 'no-task' always excludes tasks.")
+    $logger->info("Options 'no-task' and 'tasks' are both used. Be careful that 'no-task' always excludes tasks.")
         if ($self->{config}->isParamArrayAndFilled('no-task') && $self->{config}->isParamArrayAndFilled('tasks'));
 
-    foreach my $comment (@{$COMMENTS}) {
-        $self->{logger}->info($comment);
-    }
+    # install signal handler to handle graceful exit
+    $SIG{INT}  = sub { $self->terminate(); exit 0; };
+    $SIG{TERM} = sub { $self->terminate(); exit 0; };
 
-    $self->resetLastConfigLoad();
-}
-
-sub reinit {
-    my ($self) = @_;
-
-    $self->{logger}->debug('agent reinit');
-
-    $self->{config}->reloadFromInputAndBackend($self->{confdir});
-
-    my $config = $self->{config};
-
-    my $verbosity = $config->{debug} && $config->{debug} == 1 ? LOG_DEBUG  :
-                    $config->{debug} && $config->{debug} == 2 ? LOG_DEBUG2 :
-                                                                LOG_INFO   ;
-
-    my $logger = undef;
-    if (! defined($self->{logger})) {
-        $logger = FusionInventory::Agent::Logger->new(
-            config    => $config,
-            backends  => $config->{logger},
-            verbosity => $verbosity
-        );
-        $self->{logger} = $logger;
-    } else {
-        $logger = $self->{logger};
-    }
-
-    $logger->debug("Configuration directory: $self->{confdir}");
-    $logger->debug("Data directory: $self->{datadir}");
-    $logger->debug("Storage directory: $self->{vardir}");
-    $logger->debug("Lib directory: $self->{libdir}");
-
-    # handle persistent state
-    $self->_loadState();
-
-    $self->{deviceid} = _computeDeviceId() if !$self->{deviceid};
-
-    $self->_saveState();
-
-    if (!$self->getTargets()) {
-        $logger->error("No target defined, aborting");
-        exit 1;
-    }
-
-    # compute list of allowed tasks
-    my %available = $self->getAvailableTasks(disabledTasks => $config->{'no-task'});
-    my @tasks = keys %available;
-    my @plannedTasks = $self->computeTaskExecutionPlan(\@tasks);
-    $self->{tasksExecutionPlan} = \@plannedTasks;
-
-    my %available_lc = map { (lc $_) => $_ } keys %available;
-    if (!@tasks) {
-        $logger->error("No tasks available, aborting");
-        exit 1;
-    }
-
-    $logger->debug("Available tasks:");
-    foreach my $task (keys %available) {
-        $logger->debug("- $task: $available{$task}");
-    }
-    $logger->debug("Planned tasks:");
-    foreach my $task (@{$self->{tasksExecutionPlan}}) {
-        my $task_lc = lc $task;
-        $logger->debug("- $task: " . $available{$available_lc{$task_lc}});
-    }
-
-    $self->{tasks} = \@tasks;
-
-    $self->ApplyServiceOptimizations();
-
-    $self->resetLastConfigLoad();
-
-    $self->{logger}->debug('agent reinit done.');
-}
-
-sub resetLastConfigLoad {
-    my ($self) = @_;
-
-    $self->{lastConfigLoad} = time;
-}
-
-sub ApplyServiceOptimizations {
-    my ($self) = @_;
-
-    return unless ($self->{config}->{daemon} || $self->{config}->{service});
-
-    # Preload all IDS databases to avoid reload them all the time during inventory
-    if (grep { /^inventory$/i } @{$self->{tasksExecutionPlan}}) {
-        getPCIDeviceVendor(datadir => $self->{datadir});
-        getUSBDeviceVendor(datadir => $self->{datadir});
-        getEDIDVendor(datadir => $self->{datadir});
-    }
-
-    # win32 platform optimization
-    if ($OSNAME eq 'MSWin32') {
-        # Preload is64bit result to avoid a lot of WMI calls
-        FusionInventory::Agent::Tools::Win32->require();
-        FusionInventory::Agent::Tools::Win32::is64bit();
-    }
-}
-
-sub RunningServiceOptimization {
-    my ($self) = @_;
-
-    # win32 platform needs optimization
-    if ($OSNAME eq 'MSWin32') {
-        if ($self->{logger}->{verbosity} >= LOG_DEBUG) {
-            my $runmem = FusionInventory::Agent::Tools::Win32::getAgentMemorySize();
-            $self->{logger}->debug("Agent memory usage before freeing memory: $runmem");
+    if ($params{options}) {
+        foreach my $comment (@{$COMMENTS}) {
+            $self->{logger}->debug($comment);
         }
-
-        FusionInventory::Agent::Tools::Win32::FreeAgentMem();
-
-        my $current_mem = FusionInventory::Agent::Tools::Win32::getAgentMemorySize();
-        $self->{logger}->info("Agent memory usage: $current_mem");
     }
 }
 
 sub run {
     my ($self) = @_;
 
-    $self->{status} = 'waiting';
+    # API overrided in daemon or service mode
+
+    $self->setStatus('waiting');
 
     my @targets = $self->getTargets();
 
-    if ($self->{config}->{daemon} || $self->{config}->{service}) {
+    $self->{logger}->debug("Running in foreground mode");
 
-        $self->{logger}->debug2("Running in background mode");
-
-        # background mode: work on a targets list copy, but loop while
-        # the list really exists so we can stop quickly when asked for
-        while ($self->getTargets()) {
-            my $time = time();
-
-            @targets = $self->getTargets() unless @targets;
-            my $target = shift @targets;
-
-            $self->_reloadConfIfNeeded();
-
-            if ($time >= $target->getNextRunDate()) {
-
-                my $net_error = 0;
-                eval {
-                    $net_error = $self->_runTarget($target);
-                };
-                $self->{logger}->error($EVAL_ERROR) if $EVAL_ERROR;
-                if ($net_error) {
-                    # Prefer to retry early on net error
-                    $target->setNextRunDate($target->getNextRunDate()+60);
-                } else {
-                    $target->resetNextRunDate();
-                }
-
-                # Leave immediately if we passed in terminate method
-                last unless $self->getTargets();
-
-                # Call service optimization after each target run
-                $self->RunningServiceOptimization();
-            }
-
-            if ($self->{server}) {
-                # check for http interface messages, default timeout is 1 second
-                $self->{server}->handleRequests() or delay(1);
-            } else {
-                delay(1);
-            }
+    # foreground mode: check each targets once
+    my $time = time();
+    while ($self->getTargets() && @targets) {
+        my $target = shift @targets;
+        if ($self->{config}->{lazy} && $time < $target->getNextRunDate()) {
+            $self->{logger}->info(
+                "$target->{id} is not ready yet, next server contact " .
+                "planned for " . localtime($target->getNextRunDate())
+            );
+            next;
         }
-    } else {
 
-        $self->{logger}->debug2("Running in foreground mode");
+        eval {
+            $self->runTarget($target);
+        };
+        $self->{logger}->error($EVAL_ERROR) if $EVAL_ERROR;
 
-        # foreground mode: check each targets once
-        my $time = time();
-        while ($self->getTargets() && @targets) {
-            my $target = shift @targets;
-            if ($self->{config}->{lazy} && $time < $target->getNextRunDate()) {
-                $self->{logger}->info(
-                    "$target->{id} is not ready yet, next server contact " .
-                    "planned for " . localtime($target->getNextRunDate())
-                );
-                next;
-            }
-
-            eval {
-                $self->_runTarget($target);
-            };
-            $self->{logger}->error($EVAL_ERROR) if $EVAL_ERROR;
-        }
+        # Reset next run date to support --lazy option with foreground mode
+        $target->resetNextRunDate();
     }
 }
 
@@ -349,24 +167,14 @@ sub terminate {
     # Forget our targets
     $self->{targets} = [];
 
-    # Kill current running task
-    if ($self->{current_runtask}) {
-        kill 'TERM', $self->{current_runtask};
-        delete $self->{current_runtask};
-    }
-
-    $self->{logger}->info("$PROVIDER Agent exiting")
-        if $self->{config}->{daemon} || $self->{config}->{service};
-    $self->{current_task}->abort() if $self->{current_task};
-
-    # Handle sigterm callback
-    &{$self->{sigterm}}() if $self->{sigterm};
+    # Abort realtask running in that forked process or thread
+    $self->{current_task}->abort()
+        if ($self->{current_task});
 }
 
-sub _runTarget {
+sub runTarget {
     my ($self, $target) = @_;
 
-    $self->{logger}->debug('_runTarget') if defined $self->{logger};
     # the prolog dialog must be done once for all tasks,
     # but only for server targets
     my $response;
@@ -407,54 +215,31 @@ sub _runTarget {
 
     foreach my $name (@{$self->{tasksExecutionPlan}}) {
         eval {
-            $self->_runTask($target, $name, $response);
+            $self->runTask($target, $name, $response);
         };
         $self->{logger}->error($EVAL_ERROR) if $EVAL_ERROR;
-        $self->{status} = 'waiting';
+        $self->setStatus($target->paused() ? 'paused' : 'waiting');
 
         # Leave earlier while requested
         last unless $self->getTargets();
+        last if $target->paused();
     }
 
     return 0;
 }
 
-sub _runTask {
+sub runTask {
     my ($self, $target, $name, $response) = @_;
 
-    $self->{status} = "running task $name";
+    # API overrided in daemon or service mode
 
-    if ($self->{config}->{daemon} || $self->{config}->{service}) {
-        # server mode: run each task in a child process
-        if (my $pid = fork()) {
-            # parent
-            $self->{current_runtask} = $pid;
-            while (waitpid($pid, WNOHANG) == 0) {
-                if ($self->{server}) {
-                    $self->{server}->handleRequests() or delay(1);
-                } else {
-                    delay(1);
-                }
+    $self->setStatus("running task $name");
 
-                # Leave earlier while requested
-                last unless $self->getTargets();
-            }
-            delete $self->{current_runtask};
-        } else {
-            # child
-            die "fork failed: $ERRNO" unless defined $pid;
-
-            $self->{logger}->debug("forking process $PID to handle task $name");
-            $self->_runTaskReal($target, $name, $response);
-            exit(0);
-        }
-    } else {
-        # standalone mode: run each task directly
-        $self->_runTaskReal($target, $name, $response);
-    }
+    # standalone mode: run each task directly
+    $self->runTaskReal($target, $name, $response);
 }
 
-sub _runTaskReal {
+sub runTaskReal {
     my ($self, $target, $name, $response) = @_;
 
     my $class = "FusionInventory::Agent::Task::$name";
@@ -492,6 +277,23 @@ sub getStatus {
     return $self->{status};
 }
 
+sub setStatus {
+    my ($self, $status) = @_;
+
+    my $config = $self->{config};
+
+    # Rename process including status, for unix platforms
+    $0 = lc($PROVIDER) . "-agent";
+    $0 .= " (tag $config->{tag})" if $config->{tag};
+
+    if ($status) {
+        $self->{status} = $status;
+
+        # Show set status in process name on unix platforms
+        $0 .= ": $status";
+    }
+}
+
 sub getTargets {
     my ($self) = @_;
 
@@ -501,6 +303,8 @@ sub getTargets {
 sub getAvailableTasks {
     my ($self, %params) = @_;
 
+    my $logger = $self->{logger};
+
     my %tasks;
     my %disabled  = map { lc($_) => 1 } @{$params{disabledTasks}};
 
@@ -509,7 +313,7 @@ sub getAvailableTasks {
     $directory =~ s,\\,/,g;
     my $subdirectory = "FusionInventory/Agent/Task";
     # look for all Version perl modules around here
-    foreach my $file (File::Glob::glob("$directory/$subdirectory/*/Version.pm")) {
+    foreach my $file (File::Glob::bsd_glob("$directory/$subdirectory/*/Version.pm")) {
         next unless $file =~ m{($subdirectory/(\S+)/Version\.pm)$};
         my $module = file2module($1);
         my $name = file2module($2);
@@ -517,40 +321,26 @@ sub getAvailableTasks {
         next if $disabled{lc($name)};
 
         my $version;
-        if ($self->{config}->{daemon} || $self->{config}->{service}) {
-            # server mode: check each task version in a child process
-            my ($reader, $writer);
-            pipe($reader, $writer);
-            $writer->autoflush(1);
+        if (!$module->require()) {
+            $logger->debug2("module $module does not compile: $@") if $logger;
 
-            if (my $pid = fork()) {
-                # parent
-                close $writer;
-                $version = <$reader>;
-                close $reader;
-                waitpid($pid, 0);
-            } else {
-                # child
-                die "fork failed: $ERRNO" unless defined $pid;
+            # Don't keep trace of module, only really needed to fix perl 5.8 issue
+            delete $INC{module2file($module)};
 
-                close $reader;
-                $version = $self->_getTaskVersion($module);
-                print $writer $version if $version;
-                close $writer;
-                exit(0);
-            }
-        } else {
-            # standalone mode: check each task version directly
-            $version = $self->_getTaskVersion($module);
+            next;
+        }
+
+        {
+            no strict 'refs';  ## no critic
+            $version = &{$module . '::VERSION'};
         }
 
         # no version means non-functionning task
         next unless $version;
 
         $tasks{$name} = $version;
-        if (defined $self->{logger}) {
-            $self->{logger}->debug2( "getAvailableTasks() : add of task ".$name.' version '.$version );
-        }
+        $logger->debug2("getAvailableTasks() : add of task $name version $version")
+            if $logger;
     }
 
     return %tasks;
@@ -579,33 +369,34 @@ sub _getTaskVersion {
     return $version;
 }
 
-sub _isAlreadyRunning {
-    my ($self, $pidfile) = @_;
-
-    Proc::PID::File->require();
-    if ($EVAL_ERROR) {
-        $self->{logger}->debug(
-            'Proc::PID::File unavailable, unable to check for running agent'
-        );
-        return 0;
-    }
-
-    my $pid = Proc::PID::File->new();
-    $pid->{path} = $pidfile;
-    return $pid->alive();
-}
-
-sub _loadState {
+sub _handlePersistentState {
     my ($self) = @_;
 
+    # Only create storage at first call
+    unless ($self->{storage}) {
+        $self->{storage} = FusionInventory::Agent::Storage->new(
+            logger    => $self->{logger},
+            directory => $self->{vardir}
+        );
+    }
+
+    # Load current agent state
     my $data = $self->{storage}->restore(name => "$PROVIDER-Agent");
 
     $self->{deviceid} = $data->{deviceid} if $data->{deviceid};
-}
 
-sub _saveState {
-    my ($self) = @_;
+    if (!$self->{deviceid}) {
+        # compute an unique agent identifier, based on host name and current time
+        my $hostname = getHostname();
 
+        my ($year, $month , $day, $hour, $min, $sec) =
+            (localtime (time))[5, 4, 3, 2, 1, 0];
+
+        $self->{deviceid} = sprintf "%s-%02d-%02d-%02d-%02d-%02d-%02d",
+            $hostname, $year + 1900, $month + 1, $day, $hour, $min, $sec;
+    }
+
+    # Always save agent state
     $self->{storage}->save(
         name => "$PROVIDER-Agent",
         data => {
@@ -614,17 +405,6 @@ sub _saveState {
     );
 }
 
-# compute an unique agent identifier, based on host name and current time
-sub _computeDeviceId {
-    my $hostname = getHostname();
-
-    my ($year, $month , $day, $hour, $min, $sec) =
-        (localtime (time))[5, 4, 3, 2, 1, 0];
-
-    return sprintf "%s-%02d-%02d-%02d-%02d-%02d-%02d",
-        $hostname, $year + 1900, $month + 1, $day, $hour, $min, $sec;
-}
-
 sub _appendElementsNotAlreadyInList {
     my ($list, $elements, $logger) = @_;
 
@@ -706,6 +486,10 @@ sub _createTargets {
     my ($self) = @_;
 
     my $config = $self->{config};
+
+    # Always reset targets to handle re-init case
+    $self->{targets} = [];
+
     # create target list
     if ($config->{local}) {
         foreach my $path (@{$config->{local}}) {
@@ -736,82 +520,6 @@ sub _createTargets {
     }
 }
 
-sub _createDaemon {
-    my ($self) = @_;
-
-    my $config = $self->{config};
-    my $logger = $self->{logger};
-    my $pidfile  = $config->{pidfile} ||
-        $self->{vardir} . '/'.lc($PROVIDER).'.pid';
-    if ($self->_isAlreadyRunning($pidfile)) {
-        $logger->error("An agent is already running, exiting...");
-        exit 1;
-    }
-    if (!$config->{'no-fork'}) {
-
-        Proc::Daemon->require();
-        if ($EVAL_ERROR) {
-            $logger->error("Failed to load Proc::Daemon: $EVAL_ERROR");
-            exit 1;
-        }
-
-        # If we use relative path, we must stay in the current directory
-        my $workdir = substr($self->{libdir}, 0, 1) eq '/' ? '/' : getcwd();
-
-        Proc::Daemon::Init({
-                work_dir => $workdir,
-                    pid_file => $pidfile
-            });
-
-        $self->{logger}->debug("Agent daemonized");
-    }
-}
-
-sub _createHttpInterface {
-    my ($self) = @_;
-
-    my $logger = $self->{logger};
-    my $config = $self->{config};
-    FusionInventory::Agent::HTTP::Server->require();
-    if ($EVAL_ERROR) {
-        $logger->error("Failed to load HTTP server: $EVAL_ERROR");
-    } else {
-        $self->{server} = FusionInventory::Agent::HTTP::Server->new(
-            logger          => $logger,
-            agent           => $self,
-            htmldir         => $self->{datadir} . '/html',
-            ip              => $config->{'httpd-ip'},
-            port            => $config->{'httpd-port'},
-            trust           => $config->{'httpd-trust'}
-        );
-        $self->{server}->init();
-    }
-}
-
-sub _installSignalHandlers {
-    my ($self) = @_;
-
-    $SIG{INT}     = sub { $self->terminate(); exit 0; };
-    $SIG{TERM}    = sub { $self->terminate(); exit 0; };
-}
-
-sub _reloadConfIfNeeded {
-    my ($self) = @_;
-
-    if ($self->_isReloadConfNeeded()) {
-        $self->{logger}->debug2('_reloadConfIfNeeded() is true, init agent now...');
-        $self->reinit();
-    }
-}
-
-sub _isReloadConfNeeded {
-    my ($self) = @_;
-
-    my $time = time;
-    #$self->{logger}->debug2('_isReloadConfNeeded : ' . $self->{lastConfigLoad} . ' - ' . $time . ' > ' . $self->{config}->{'conf-reload-interval'} . ' ?');
-    return ($self->{config}->{'conf-reload-interval'} > 0) && (($time - $self->{lastConfigLoad}) > $self->{config}->{'conf-reload-interval'});
-}
-
 1;
 __END__
 
@@ -866,6 +574,10 @@ Terminate the agent.
 
 Get the current agent status.
 
+=head2 setStatus()
+
+Set new agent status, also updates process name on unix platforms.
+
 =head2 getTargets()
 
 Get all targets.
