@@ -41,7 +41,7 @@ sub _init {
     $self->_loadState();
 
     $self->{nextRunDate} = $self->_computeNextRunDate()
-        if !$self->{nextRunDate};
+        if (!$self->{nextRunDate} || $self->{nextRunDate} < time-$self->getMaxDelay());
 
     $self->_saveState();
 
@@ -66,10 +66,26 @@ sub setNextRunDate {
     $self->_saveState();
 }
 
+sub setNextRunDateFromNow {
+    my ($self, $nextRunDelay) = @_;
+
+    lock($self->{nextRunDate}) if $self->{shared};
+    if ($nextRunDelay) {
+        # While using nextRunDelay, we double it on each consecutive call until
+        # delay reach target defined maxDelay. This is only used on network failure.
+        $nextRunDelay = 2 * $self->{_nextrundelay} if ($self->{_nextrundelay});
+        $nextRunDelay = $self->getMaxDelay() if ($nextRunDelay > $self->getMaxDelay());
+        $self->{_nextrundelay} = $nextRunDelay;
+    }
+    $self->{nextRunDate} = time + ($nextRunDelay || 0);
+    $self->_saveState();
+}
+
 sub resetNextRunDate {
     my ($self) = @_;
 
     lock($self->{nextRunDate}) if $self->{shared};
+    $self->{_nextrundelay} = 0;
     $self->{nextRunDate} = $self->_computeNextRunDate();
     $self->_saveState();
 }
@@ -80,6 +96,24 @@ sub getNextRunDate {
     return $self->{nextRunDate};
 }
 
+sub paused {
+    my ($self) = @_;
+
+    return $self->{_paused} || 0;
+}
+
+sub pause {
+    my ($self) = @_;
+
+    $self->{_paused} = 1;
+}
+
+sub continue {
+    my ($self) = @_;
+
+    delete $self->{_paused};
+}
+
 sub getFormatedNextRunDate {
     my ($self) = @_;
 
@@ -187,6 +221,10 @@ Get nextRunDate attribute as a formated string.
 
 Set next execution date.
 
+=head2 setNextRunDateFromNow($nextRunDelay)
+
+Set next execution date from now and after $nextRunDelay seconds (0 by default).
+
 =head2 resetNextRunDate()
 
 Set next execution date to a random value.
