@@ -174,6 +174,9 @@ sub processRemote {
             msg    => 'all checks are ok'
         );
 
+        # USER INTERACTION
+        next if $job->next_on_usercheck(type => 'before');
+
         $logger->debug2("Downloading for job $job->{uuid}...");
 
         # DOWNLOADING
@@ -235,6 +238,9 @@ sub processRemote {
                     redo FETCHFILE;
                 } else { # Give up...
 
+                    # USER INTERACTION after download failure
+                    $job->next_on_usercheck(type => 'after_download_failure');
+
                     $job->setStatus(
                         file   => $file,
                         status => 'ko',
@@ -252,10 +258,16 @@ sub processRemote {
             msg    => 'success'
         );
 
+        # USER INTERACTION after download
+        next if $job->next_on_usercheck(type => 'after_download');
+
         $logger->debug2("Preparation for job $job->{uuid}...");
 
         $job->currentStep('prepare');
         if (!$workdir->prepare()) {
+            # USER INTERACTION on preparation failure
+            $job->next_on_usercheck(type => 'after_failure');
+
             $job->setStatus(
                 status => 'ko',
                 msg    => 'failed to prepare work dir'
@@ -296,16 +308,37 @@ sub processRemote {
             eval { $ret = $actionProcessor->process($actionName, $params, $logger); };
             $ret->{msg} = [] unless $ret->{msg};
             push @{$ret->{msg}}, $@ if $@;
-            if ( !$ret->{status} ) {
+
+            my $name = $params->{name} || "action #".($actionnum+1);
+
+            # Log msg lines: can be heavy while running a command with high logLineLimit parameter
+            my $logLineLimit = defined($params->{logLineLimit}) ?
+                $params->{logLineLimit} : 10 ;
+
+            # Really report nothing to server if logLineLimit=0 & status is ok
+            $ret->{msg} = [] if (!$logLineLimit && $ret->{status});
+
+            # Add 7 to always output header & retCode analysis lines for cmd command, unless in nolimit (-1)
+            $logLineLimit += 7 unless ($logLineLimit < 0);
+
+            foreach my $line (@{$ret->{msg}}) {
+                next unless ($line);
                 $job->setStatus(
-                    msg       => $ret->{msg},
+                    msg       => "$name: $line",
                     actionnum => $actionnum,
                 );
+                last unless --$logLineLimit;
+            }
+
+            if ( !$ret->{status} ) {
+
+                # USER INTERACTION after action failure
+                $job->next_on_usercheck(type => 'after_failure');
 
                 $job->setStatus(
                     status    => 'ko',
                     actionnum => $actionnum,
-                    msg       => "action #".($actionnum+1)." processing failure"
+                    msg       => "$name, processing failure"
                 );
 
                 next JOB;
@@ -313,12 +346,15 @@ sub processRemote {
             $job->setStatus(
                 status    => 'ok',
                 actionnum => $actionnum,
-                msg       => "action #".($actionnum+1)." processing success"
+                msg       => "$name, processing success"
             );
 
             $actionnum++;
         }
 
+        # USER INTERACTION
+        $job->next_on_usercheck(type => 'after');
+
         $logger->debug2("Finished job $job->{uuid}...");
 
         $job->currentStep('end');
