@@ -204,8 +204,13 @@ sub _initModulesList {
         no strict 'refs'; ## no critic (ProhibitNoStrict)
         $self->{modules}->{$module}->{runAfter} = [
             $parent ? $parent : (),
-            ${$module . '::runAfter'} ? @${$module . '::runAfter'} : ()
+            ${$module . '::runAfter'} ? @${$module . '::runAfter'} : (),
+            ${$module . '::runAfterIfEnabled'} ? @${$module . '::runAfterIfEnabled'} : ()
         ];
+        $self->{modules}->{$module}->{runAfterIfEnabled} = {
+            map { $_ => 1 }
+                ${$module . '::runAfterIfEnabled'} ? @${$module . '::runAfterIfEnabled'} : ()
+        };
     }
 
     # second pass: disable fallback modules
@@ -248,8 +253,15 @@ sub _runModule {
         die "module $other_module, needed before $module, not found"
             if !$self->{modules}->{$other_module};
 
-        die "module $other_module, needed before $module, not enabled"
-            if !$self->{modules}->{$other_module}->{enabled};
+        if (!$self->{modules}->{$other_module}->{enabled}) {
+            if ($self->{modules}->{$module}->{runAfterIfEnabled}->{$other_module}) {
+                # soft dependency: run current module without required one
+                next;
+            } else {
+                # hard dependency: abort current module execution
+                die "module $other_module, needed before $module, not enabled";
+            }
+        }
 
         die "circular dependency between $module and $other_module"
             if $self->{modules}->{$other_module}->{used};
