@@ -44,9 +44,9 @@ sub _getSoftwaresList {
     }
     my $localTimeOffset = FusionInventory::Agent::Tools::MacOS::detectLocalTimeOffset();
     $infos = FusionInventory::Agent::Tools::MacOS::getSystemProfilerInfos(
+        %params,
         type            => 'SPApplicationsDataType',
-        localTimeOffset => $localTimeOffset,
-        @_
+        localTimeOffset => $localTimeOffset
     );
 
     my $info = $infos->{Applications};
@@ -65,19 +65,43 @@ sub _getSoftwaresList {
             $formattedDate = _formatDate($formattedDate, $logger)
         }
 
+        my ($category, $userName) = _extractSoftwareSystemCategoryAndUserName($app->{'Location'});
         push @softwares, {
             NAME      => $name,
             VERSION   => $app->{'Version'},
             COMMENTS  => $app->{'Kind'} ? '[' . $app->{'Kind'} . ']' : undef,
             PUBLISHER => $app->{'Get Info String'},
             # extract date's data and format these data
-            INSTALLDATE => $formattedDate
+            INSTALLDATE => $formattedDate,
+            SYSTEM_CATEGORY => $category,
+            USERNAME => $userName
         };
     }
 
     return \@softwares;
 }
 
+sub _extractSoftwareSystemCategoryAndUserName {
+    my ($str) = @_;
+
+    my $category = '';
+    my $userName = '';
+    return ($category, $userName) unless $str;
+
+    if ($str =~ /^\/Users\/([^\/]+)\/([^\/]+\/[^\/]+)\//
+        || $str =~ /^\/Users\/([^\/]+)\/([^\/]+)\//) {
+        $userName = $1;
+        $category = $2 if $2 !~ /^Downloads|^Desktop/;
+    } elsif ($str =~ /^\/Volumes\/[^\/]+\/([^\/]+\/[^\/]+)\//
+        || $str =~ /^\/Volumes\/[^\/]+\/([^\/]+)\//
+        || $str =~ /^\/([^\/]+\/[^\/]+)\//
+        || $str =~ /^\/([^\/]+)\//) {
+        $category = $1;
+    }
+
+    return ($category, $userName);
+}
+
 sub _formatDate {
     my ($dateStr, $logger) = @_;
 
