@@ -5,8 +5,6 @@ use warnings;
 
 use FusionInventory::Agent::Tools::Win32;
 
-my $seen;
-
 sub isEnabled {
     my (%params) = @_;
     return 0 if $params{no_category}->{video};
@@ -16,6 +14,7 @@ sub isEnabled {
 sub doInventory {
     my (%params) = @_;
 
+    my %seen;
     my $inventory = $params{inventory};
 
     foreach my $object (getWMIObjects(
@@ -43,7 +42,7 @@ sub doInventory {
             if $video->{MEMORY};
 
         # avoid duplicates
-        next if $seen->{$video->{NAME}}++;
+        next if $seen{$video->{NAME}}++;
 
         $inventory->addEntry(
             section => 'VIDEOS',
