@@ -31,6 +31,7 @@ our @EXPORT = qw(
     getCanonicalInterfaceSpeed
     getCanonicalSize
     getSanitizedString
+    getUtf8String
     trimWhitespace
     getFirstLine
     getFirstMatch
@@ -134,6 +135,10 @@ sub getCanonicalManufacturer {
         $manufacturer = "Seagate";
     } elsif ($manufacturer =~ /^(HD|IC|HU|HGST)/) {
         $manufacturer = "Hitachi";
+    } elsif ($manufacturer =~ /^APPLE/i) {
+        $manufacturer = "Apple";
+    } elsif ($manufacturer =~ /^OPTIARC/i) {
+        $manufacturer = "Sony";
     }
 
     return $manufacturer;
@@ -146,16 +151,21 @@ sub getCanonicalSpeed {
 
     return undef unless $speed;
 
+    return $speed if $speed =~ /^([,.\d]+)$/;
+
     return 400 if $speed =~ /^PC3200U/;
 
-    return undef unless $speed =~ /^([,.\d]+) \s? (\S+)$/x;
+    return undef unless $speed =~ /^([,.\d]+) \s? (\S+)/x;
     my $value = $1;
     my $unit = lc($2);
 
+    # Remark: we need to return speed in MHz. Even if MT/s is not accurately
+    # equivalent to MHz, we nned to return the value so server can extract it
     return
-        $unit eq 'ghz' ? $value * 1000 :
-        $unit eq 'mhz' ? $value        :
-                         undef         ;
+        $unit eq 'ghz'  ? $value * 1000 :
+        $unit eq 'mhz'  ? $value        :
+        $unit eq 'mt/s' ? $value        :
+                          undef         ;
 }
 
 sub getCanonicalInterfaceSpeed {
@@ -192,6 +202,7 @@ sub getCanonicalSize {
     return undef unless $size =~ /^([,.\d]+) (\S+)$/x;
     my $value = $1;
     my $unit = lc($2);
+    $value =~ s/,/\./;
 
     return
         $unit eq 'tb'    ? $value * $base * $base        :
@@ -220,14 +231,11 @@ sub compareVersion {
         );
 }
 
-sub getSanitizedString {
+sub getUtf8String {
     my ($string) = @_;
 
     return unless defined $string;
 
-    # clean control caracters
-    $string =~ s/[[:cntrl:]]//g;
-
     # encode to utf-8 if needed
     if (!Encode::is_utf8($string) && $string !~ m/\A(
           [\x09\x0A\x0D\x20-\x7E]           # ASCII
@@ -245,6 +253,17 @@ sub getSanitizedString {
     return $string;
 }
 
+sub getSanitizedString {
+    my ($string) = @_;
+
+    return unless defined $string;
+
+    # clean control caracters
+    $string =~ s/[[:cntrl:]]//g;
+
+    return getUtf8String($string);
+}
+
 sub trimWhitespace {
     my ($value) = @_;
     $value =~ s/^\s+//;
@@ -402,7 +421,7 @@ sub hex2char {
     my ($value) = @_;
 
     ## no critic (ExplicitReturnUndef)
-    return undef unless $value;
+    return undef unless defined $value;
     return $value unless $value =~ /^0x/;
 
     $value =~ s/^0x//; # drop hex prefix
@@ -413,7 +432,7 @@ sub hex2dec {
     my ($value) = @_;
 
     ## no critic (ExplicitReturnUndef)
-    return undef unless $value;
+    return undef unless defined $value;
     return $value unless $value =~ /^0x/;
 
     return oct($value);
@@ -423,7 +442,7 @@ sub dec2hex {
     my ($value) = @_;
 
     ## no critic (ExplicitReturnUndef)
-    return undef unless $value;
+    return undef unless defined $value;
     return $value if $value =~ /^0x/;
 
     return sprintf("0x%x", $value);
@@ -589,6 +608,10 @@ Returns a normalized size value (in Mb) for given one.
 Returns the input stripped from any control character, properly encoded in
 UTF-8.
 
+=head2 getUtf8String($string)
+
+Returns the input properly encoded in UTF-8.
+
 =head2 trimWhitespace($string)
 
 Remove leading and trailing whitespace, and fold multiple whitespace
