@@ -79,6 +79,11 @@ sub getDmidecodeInfos {
     }
     close $handle;
 
+    # push last block in list if still defined
+    if ($block) {
+        push(@{$info->{$type}}, $block);
+    }
+
     # do not return anything if dmidecode output is obviously truncated
     return if keys %$info < 2;
 
@@ -107,7 +112,7 @@ sub getCpusFromDmidecode {
         my $cpu = {
             SERIAL       => $info->{'Serial Number'},
             ID           => $info->{ID},
-            CORE         => $info->{'Core Count'} || $info->{'Core Enabled'},
+            CORE         => $info->{'Core Enabled'} || $info->{'Core Count'},
             THREAD       => $info->{'Thread Count'},
             FAMILYNAME   => $info->{'Family'},
             MANUFACTURER => $manufacturer
@@ -153,6 +158,12 @@ sub getCpusFromDmidecode {
             }
         }
 
+        # Add CORECOUNT if we have less enabled cores than total count
+        if ($info->{'Core Enabled'} && $info->{'Core Count'}) {
+            $cpu->{CORECOUNT} = $info->{'Core Count'}
+                unless ($info->{'Core Enabled'} == $info->{'Core Count'});
+        }
+
         push @cpus, $cpu;
     }
 
